/*
 * Copyright (c) 2020, Max Klein
 *
 * This file is part of the tree-finder library, distributed under the terms of
 * the BSD 3 Clause license. The full license can be found in the LICENSE file.
 */
import { DEFAULT_COL } from "./content";
const SORT_ORDERS = ["asc", "desc", null];
const DEFAULT_SORT_ORDER = SORT_ORDERS[0];
export class FilterPatterns {
    constructor(patterns) {
        this._patterns = {};
        if (!patterns) {
            return;
        }
        for (const fpat of patterns) {
            this.set(fpat);
        }
    }
    set(fpat) {
        const { col, pattern } = fpat;
        if (!pattern) {
            delete this._patterns[col];
        }
        else {
            this._patterns[col] = pattern;
        }
    }
    get any() {
        return !!Object.keys(this.patterns).length;
    }
    get patterns() {
        return this._patterns;
    }
}
export class SortStates {
    constructor(states) {
        this.defaultColumn = DEFAULT_COL;
        states = states !== null && states !== void 0 ? states : [{ col: DEFAULT_COL, order: DEFAULT_SORT_ORDER }];
        this.states = states.map((state, ix) => {
            return {
                ...state,
                ix,
                sign: state.order === "desc" ? -1 : 1,
            };
        });
        this.byColumn = this.states.reduce((obj, state) => { obj[state.col] = state; return obj; }, {});
    }
}
function contentsFiltererClosure(filterPatterns, pathDepth = 0) {
    return function contentsFilterer(content) {
        for (const key of Object.keys(filterPatterns.patterns)) {
            if (key === "path") {
                if (!content.getPathAtDepth(pathDepth).join("/").includes(filterPatterns.patterns[key])) {
                    return false;
                }
            }
            else {
                if (!content.row[key].includes(filterPatterns.patterns[key])) {
                    return false;
                }
            }
        }
        return true;
    };
}
function contentsSorterClosure(sortStates) {
    return function contentsSorter(l, r) {
        for (const { col, sign } of sortStates.states) {
            let cmp;
            let lval = l.row[col];
            let rval = r.row[col];
            // the "paths" field will be an array of the parts of the path, just compare the last part
            if (Array.isArray(lval)) {
                lval = lval[lval.length - 1];
                rval = rval[rval.length - 1];
            }
            if (typeof lval === "string") {
                cmp = lval.localeCompare(rval);
            }
            else {
                cmp = lval - rval;
            }
            if (cmp) {
                return sign * cmp;
            }
        }
        return 0;
    };
}
function updateSort(sortStates, col, multisort = false) {
    let newStates = [...sortStates.states];
    if (col in sortStates.byColumn) {
        const { ix: currentIdx, order: oldOrder } = sortStates.byColumn[col];
        const order = SORT_ORDERS[(SORT_ORDERS.indexOf(oldOrder) + 1) % SORT_ORDERS.length];
        if (order) {
            // update the sort_dir
            newStates[currentIdx] = { col, order };
        }
        else {
            // remove this column from the sort
            newStates.splice(currentIdx, 1);
        }
    }
    else {
        const newState = { col, order: DEFAULT_SORT_ORDER };
        if (multisort) {
            newStates.push(newState);
        }
        else {
            newStates = [newState];
        }
    }
    return new SortStates(newStates);
}
async function _flattenContents(content, filterer, sorter, contentsFlat = []) {
    if (content.cache) {
        let children;
        if (sorter) {
            children = content.cache.sort(sorter);
        }
        else {
            children = content.cache;
        }
        for (const child of children) {
            contentsFlat.push(child);
            if (child.isExpand) {
                await _flattenContents(child, filterer, sorter, contentsFlat);
            }
        }
    }
    if (filterer) {
        return contentsFlat.filter(filterer);
    }
    else {
        return contentsFlat;
    }
}
export async function filterContentRoot({ root, filterPatterns, pathDepth }) {
    // get sorter then sort/flatten any expanded children of root
    return await _flattenContents(root, contentsFiltererClosure(filterPatterns, pathDepth !== null && pathDepth !== void 0 ? pathDepth : root.row.path.length), undefined, []);
}
export async function filterSortContentRoot({ root, filterPatterns, sortStates, col, multisort, pathDepth }) {
    // update sort orders, if requested
    if (col) {
        sortStates = updateSort(sortStates, col, multisort);
        if (!sortStates.states.length) {
            // if empty, use default sortStates
            sortStates = new SortStates();
        }
    }
    // get sorter then sort/flatten any expanded children of root
    return [await _flattenContents(root, contentsFiltererClosure(filterPatterns, pathDepth !== null && pathDepth !== void 0 ? pathDepth : root.row.path.length), contentsSorterClosure(sortStates), []), sortStates];
}
export async function sortContentRoot({ root, sortStates, col, multisort }) {
    // update sort orders, if requested
    if (col) {
        sortStates = updateSort(sortStates, col, multisort);
        if (!sortStates.states.length) {
            // if empty, use default sortStates
            sortStates = new SortStates();
        }
    }
    // get sorter then sort/flatten any expanded children of root
    return [await _flattenContents(root, undefined, contentsSorterClosure(sortStates), []), sortStates];
}
//# sourceMappingURL=filtersort.js.map