/*
 * Copyright (c) 2020, Max Klein
 *
 * This file is part of the tree-finder library, distributed under the terms of
 * the BSD 3 Clause license. The full license can be found in the LICENSE file.
 */
import { BehaviorSubject } from "rxjs";
import { Content } from "./content";
import { filterContentRoot, FilterPatterns, filterSortContentRoot, SortStates } from "./filtersort";
class CrumbModel {
    constructor() {
        this._crumbsRows = [];
        this._crumbsRows = [];
        this.crumbNamesSub = new BehaviorSubject([]);
        this.revertedCrumbSub = new BehaviorSubject(undefined);
    }
    revert(crumbIx) {
        this._crumbsRows = this._crumbsRows.slice(0, crumbIx + 1);
        const last = this._crumbsRows.pop();
        this.revertedCrumbSub.next(last);
        return last;
    }
    extend(crumbs) {
        this._crumbsRows = [...this._crumbsRows, ...crumbs];
        this._onCrumbUpdate();
        const last = this._crumbsRows[this._crumbsRows.length - 1];
        return last;
    }
    async _onCrumbUpdate() {
        this.crumbNamesSub.next(this._crumbsRows.map(r => r.path[r.path.length - 1]));
    }
}
export class ContentsModel {
    constructor(root, options = {}) {
        this.columnWidthsSub = new BehaviorSubject([]);
        this.drawSub = new BehaviorSubject(false);
        this.selection = new SelectionModel();
        this.crumbs = new CrumbModel();
        this._filterPatterns = new FilterPatterns();
        this._parentMap = new WeakMap();
        this._sortStates = new SortStates();
        this.options = options;
        this.crumbs.revertedCrumbSub.subscribe(async (x) => {
            if (x) {
                await this.open(x);
                this.requestDraw(true);
            }
        });
        this.open(root);
    }
    async open(root) {
        this._contents = [];
        const newCrumbs = [root];
        while (this._parentMap.has(newCrumbs[0])) {
            newCrumbs.unshift(this._parentMap.get(newCrumbs[0]));
        }
        this.crumbs.extend(newCrumbs);
        this._parentMap = new WeakMap();
        this._root = new Content(root);
        this.initColumns();
        await this._root.expand();
        // sort the root's contents and display
        await this.sort({ autosize: true });
    }
    initColumns() {
        var _a;
        const columns = (_a = this.options.columnNames) !== null && _a !== void 0 ? _a : Object.keys(this._root.row);
        this._columns = columns.filter(x => !["path", "getChildren"].includes(x));
        // build a columnName -> columnIx table; add 1 to account for leading "path" column
        this._ixByColumn = Object.fromEntries(this._columns.map((x, ix) => [x, ix + 1]));
    }
    // splice out the contents of the collapsed node and any expanded subnodes
    async collapse(rix) {
        const content = this._contents[rix];
        let check_ix = rix + 1;
        let npop = 0;
        while (this._contents[check_ix] && this._contents[check_ix].row.path.length > content.row.path.length) {
            check_ix++;
            npop++;
        }
        this._contents.splice(rix + 1, npop);
        content.close();
        this.requestDraw(true);
    }
    async expand(rix) {
        var _a;
        const content = this._contents[rix];
        content.expand();
        for (const child of (_a = await (content === null || content === void 0 ? void 0 : content.getChildren())) !== null && _a !== void 0 ? _a : []) {
            this._parentMap.set(child.row, content.row);
        }
        const [nodeContents, _] = await filterSortContentRoot({ root: content, filterPatterns: this._filterPatterns, sortStates: this._sortStates, pathDepth: this.pathDepth });
        this._contents.splice(rix + 1, 0, ...nodeContents);
        this.requestDraw();
    }
    async filter(props = {}) {
        const { autosize = true } = props;
        await this._root.expand();
        this._contents = await filterContentRoot({ root: this._root, filterPatterns: this._filterPatterns });
        this.requestDraw(autosize);
    }
    onFilterInput(fpat) {
        this._filterPatterns.set(fpat);
        this.filter();
    }
    requestDraw(autosize = false) {
        this.drawSub.next(autosize);
    }
    async sort(props = {}) {
        const { col, multisort, autosize = false } = props;
        await this._root.expand();
        [this._contents, this._sortStates] = await filterSortContentRoot({ root: this._root, filterPatterns: this._filterPatterns, sortStates: this._sortStates, col, multisort });
        this.requestDraw(autosize);
    }
    get columns() {
        return this._columns;
    }
    get contents() {
        return this._contents;
    }
    get filterPatterns() {
        return this._filterPatterns;
    }
    get pathDepth() {
        return this._root.row.path.length;
    }
    get sortStates() {
        return this._sortStates;
    }
    get root() {
        return this._root;
    }
    get options() {
        return { ...this._options };
    }
    set options(options) {
        const { columnNames, doRefetch = false, } = options;
        this._options = {
            columnNames: columnNames,
            doRefetch,
        };
    }
    get ixByColumn() {
        return this._ixByColumn;
    }
}
export class SelectionModel {
    constructor() {
        this.clear();
    }
    clear() {
        this.pivot = null;
        this.range = [];
        this.selection = new Set();
    }
    get(contents) {
        const selected = [];
        for (const content of contents) {
            if (this.has(content)) {
                selected.push(content);
            }
        }
        return selected;
    }
    has(content) {
        return this.selection.has(content.row.path.join("/"));
    }
    select(content, add = false) {
        if (!add) {
            this.clear();
        }
        else {
            this.range = [];
        }
        const pathstr = content.row.path.join("/");
        if (this.selection.has(pathstr)) {
            this.selection.delete(pathstr);
        }
        else {
            this.selection.add(pathstr);
        }
        this.pivot = this.selection.size > 0 ? pathstr : null;
    }
    selectRange(end, contents) {
        for (const pathstr of this.range) {
            // pivot around any existing range
            this.selection.delete(pathstr);
        }
        this.range = SelectionModel.findRange(this.pivot, end.row.path.join("/"), contents.map(c => c.row.path.join("/")));
        for (const pathstr of this.range) {
            this.selection.add(pathstr);
        }
    }
}
(function (SelectionModel) {
    function findRange(start, end, vals) {
        if (start === end) {
            return [];
        }
        const pivotIx = start !== null ? vals.indexOf(start) : -1;
        const endIx = vals.indexOf(end);
        const range = [];
        if (pivotIx > endIx) {
            for (let i = endIx; i < pivotIx; i++) {
                range.push(vals[i]);
            }
        }
        else {
            for (let i = pivotIx + 1; i <= endIx; i++) {
                range.push(vals[i]);
            }
        }
        return range;
    }
    SelectionModel.findRange = findRange;
})(SelectionModel || (SelectionModel = {}));
//# sourceMappingURL=model.js.map